package de.uni_passau.fim;

import java.util.HashMap;
import java.util.Map;

import de.uni_passau.fim.gui.ColorRadioButton;

import android.app.Activity;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;

/**
 * This class lets the user input a challenge and reuses large parts of the code
 * in MasterMind.java
 * 
 */
public class Challenge extends Activity implements View.OnClickListener {

	// the currently selected color
	private int currentColor;

	// the challenge the user has selected
	private int challenge[];

	// a map structure to efficiently find the row on which was clicked
	private Map<ImageView, Integer> currentCols;

	// the button we use to acknowledge our selection
	private Button okButton;

	// a static reference to the calling multiplayer class
	private static Multiplayer mpClass;

	/**
	 * Getter for mpClass
	 * 
	 * @return Multiplayer
	 */
	public static Multiplayer getMpClass() {
		return mpClass;
	}

	/**
	 * Setter for mpClass
	 * 
	 * @param mpClass
	 *            the calling Multiplayer class instance
	 */
	public static void setMpClass(Multiplayer mpClass) {
		Challenge.mpClass = mpClass;
	}

	/**
	 * Set the current color depending on which radio button we click
	 */
	private OnCheckedChangeListener selection_listener = new OnCheckedChangeListener() {

		public void onCheckedChanged(RadioGroup group, int checkedId) {
			View v = findViewById(checkedId);
			if (v instanceof ColorRadioButton) {
				currentColor = ((ColorRadioButton) v).getColor();
			}
		}

	};

	/**
	 * Here we store which circle has which color
	 */
	private OnClickListener set_listener = new OnClickListener() {

		public void onClick(View v) {
			if (v instanceof ImageView) {
				setColor((ImageView) v, currentColor);
				challenge[currentCols.get((ImageView) v)] = currentColor;
				// we also verify that all colors have been set
				// before we activate the okButton
				okButton.setEnabled(true);
				for (int i = 0; i < challenge.length; ++i) {
					if (challenge[i] == 0) {
						okButton.setEnabled(false);
						break;
					}
				}
			}
		}

	};

	/**
	 * Helper function to change color of an imageview
	 * 
	 * @param image
	 *            the circle
	 * @param color
	 *            the new color
	 */
	protected static void setColor(ImageView image, int color) {
		Drawable d = image.getDrawable();
		if (d instanceof GradientDrawable) {
			GradientDrawable gd = (GradientDrawable) image.getDrawable();
			gd.mutate();
			gd.setColor(color);
			gd.invalidateSelf();
		}
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.challengeinput);

		currentCols = new HashMap<ImageView, Integer>();
		currentColor = -1;

		RadioGroup rg = (RadioGroup) this.findViewById(R.id.ChallengeSelection);
		rg.setOnCheckedChangeListener(selection_listener);

		int[] colors = new int[rg.getChildCount()];
		for (int i = 0; i < rg.getChildCount(); i++) {
			View v = rg.getChildAt(i);
			if (v instanceof ColorRadioButton) {
				colors[i] = ((ColorRadioButton) v).getColor();
			}
		}
		Multiplayer.setColors(colors);
		
		ViewGroup current = (ViewGroup) this
				.findViewById(R.id.CurrentChallengeRow);

		okButton = (Button) this.findViewById(R.id.okButton);
		okButton.setOnClickListener(this);

		int childCount = current.getChildCount();
		challenge = new int[childCount];

		for (int i = 0; i < childCount; ++i) {
			View v = current.getChildAt(i);
			if (v instanceof ImageView) {
				v.setOnClickListener(set_listener);
				currentCols.put((ImageView) v, i);
			}
		}

	}

	/**
	 * Called when we click the okButton
	 */
	public void onClick(View v) {
		// now we set the static class member in Multiplayer.java
		Multiplayer.setChallengeInt(challenge);
		mpClass.signalReady();
		finish();

	}
}
