package de.uni_passau.fim;

import java.util.Collection;
import java.util.HashMap;

import de.uni_passau.fim.R;
import de.uni_passau.fim.R.id;
import de.uni_passau.fim.R.layout;
import de.uni_passau.fim.evaluator.Mastermind;
import de.uni_passau.fim.evaluator.MyEvaluator;
import de.uni_passau.fim.evaluator.MyEvaluator.BoardInfo;
import de.uni_passau.fim.evaluator.MyEvaluator.Stats;
import de.uni_passau.fim.strategies.IStrategy;
import de.uni_passau.fim.strategies.StrategyConfiguration;
import de.uni_passau.fim.strategies.StrategyFactory;
import de.uni_passau.fim.strategies.Utils;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.TextView;

public class Evaluator extends Activity {

	private EditText mNumOfColors;
	private EditText mNumOfPlaces;
	private EditText mNumOfRows;
	
	private CheckBox mWithRepetition;
	private Spinner mStrategySpinner;
	
	private RadioButton mRunModeRandom;
	private EditText mNumOfRandomGames;

	private EditText mResults;
	
	ProgressDialog progressDialog;
	ProgressThread progressThread;
	AlertDialog invalidConfAlertDialog;

	MyEvaluator mE;
	private ArrayAdapter<CharSequence> mStrategySpinnerAdapter;

	private HashMap<String, Object> mCurrentSpecialStrategyConf;
	
	
	static final int PROGRESS_DIALOG = 0;
	static final int INVALID_CONFIGURATION = 1;
	protected static final int REQ_CODE_STRATEGY_CONFIG = 0;
	protected static final String INTENT_DATA_NAMES = "de.uni_passau.fim.EvaluatorConfigNames";
	protected static final String INTENT_DATA_VALUES = "de.uni_passau.fim.EvaluatorConfigValues";
	
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.evaluator);
		
		initStrategySelection();
		initConfigFields();
		initResultView();
		initButtonListeners();
		
		
	}
	
	private void initStrategySelection() {
		String[] stratNames = StrategyFactory.getInstance().getUniqueStrategyNames();
		mStrategySpinner = (Spinner)findViewById(R.id.evalStrategySpinner);
		mStrategySpinnerAdapter = new ArrayAdapter<CharSequence>(this, android.R.layout.simple_spinner_item, stratNames);
		mStrategySpinnerAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		mStrategySpinner.setAdapter(mStrategySpinnerAdapter);
		mStrategySpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

			public void onItemSelected(AdapterView<?> arg0, View arg1,
					int arg2, long arg3) {
				IStrategy s = getSelectedStrategy();
				System.out.println("Changed strategy to " + s.getUniqueName());
				mCurrentSpecialStrategyConf = s.getDefaultSpecialConfig();
			}

			public void onNothingSelected(AdapterView<?> arg0) {
			}
		});
		mCurrentSpecialStrategyConf = getSelectedStrategy().getDefaultSpecialConfig();
	}
	
	private void initConfigFields() {
		mNumOfColors = (EditText)findViewById(R.id.editTextEvalNumOfColors);
		mNumOfPlaces = (EditText)findViewById(R.id.editTextEvalNumOfPlaces);
		mNumOfRows = (EditText)findViewById(R.id.editTextEvalNumOfRows);
		mWithRepetition = (CheckBox)findViewById(R.id.checkBoxEvalWithRepetition);
		mRunModeRandom = (RadioButton)findViewById(R.id.radioRunModeRandom);
		mNumOfRandomGames = (EditText)findViewById(R.id.editTextEvalNumOfRandomGames);
	}
	
	private void initResultView() {
		mResults = (EditText)findViewById(R.id.editTextResult);
	}
	
	private void initButtonListeners() {
		OnClickListener l = new OnClickListener() {
			
			public void onClick(View v) {
				if (!isValidData())
					return;
				Mastermind m = gatherMastermindConfig();
				IStrategy s = gatherStrategyConfig(m);
				MyEvaluator e = gatherEvaluatorConfig(m, s);
				startEvaluation(e);
			}
		};
		((Button)findViewById(R.id.evalStartButton)).setOnClickListener(l);
		
		l = new OnClickListener() {
			
			public void onClick(View v) {
				Intent evaluationIntent = new Intent();
				evaluationIntent.setClassName("de.uni_passau.fim", "de.uni_passau.fim.StrategyConfig");
				setConfigDataForIntent(evaluationIntent);
				startActivityForResult(evaluationIntent, REQ_CODE_STRATEGY_CONFIG);
			}
		};
		((Button)findViewById(R.id.evalStratConfButton)).setOnClickListener(l);
	}
	
	private boolean isValidData() {
		return validateText(mNumOfColors) 
			&& validateText(mNumOfPlaces) 
			&& validateText(mNumOfRows) 
			&& validateText(mNumOfRandomGames);
	}
	
	private boolean validateText(EditText v) {
		try {
			Integer.parseInt(v.getText().toString());
		} catch (Exception e) {
			v.requestFocus();
			return false;
		}
		return true;
	}
	
	private void setConfigDataForIntent(Intent evaluationIntent) {
		// IStrategy s = getSelectedStrategy();
		// HashMap<String, Object> dsc = s.getDefaultSpecialConfig();
		String[] names;
		int[] values;
		if (mCurrentSpecialStrategyConf == null || mCurrentSpecialStrategyConf.size() == 0) {
			names = new String[0];			
			values = new int[0];
		} else {
			int size = mCurrentSpecialStrategyConf.size();
			names = new String[size];
			values = new int[size];
			names = mCurrentSpecialStrategyConf.keySet().toArray(names);
			Collection<Object> vs = mCurrentSpecialStrategyConf.values();
			int i = 0;
			for (Object object : vs) {
				int tmp = (Integer)object;
				values[i++] = tmp;
			}
		}
		evaluationIntent.putExtra(INTENT_DATA_NAMES, names);
		evaluationIntent.putExtra(INTENT_DATA_VALUES, values);
		
		
	}
	
	private Mastermind gatherMastermindConfig() {
		int numOfColors =  Integer.parseInt(mNumOfColors.getText().toString());
		int numOfPlaces = Integer.parseInt(mNumOfPlaces.getText().toString());
		int numOfRows = Integer.parseInt(mNumOfRows.getText().toString());
		boolean withRepetition = mWithRepetition.isChecked();
		System.out.println("Repetition is checked? " + withRepetition);
		return new Mastermind(numOfColors, numOfPlaces, withRepetition, numOfRows);
	}
	
	private IStrategy getSelectedStrategy() {
		String strategy = (String)mStrategySpinner.getSelectedItem();
		System.out.println("Using strategy " + strategy);
		IStrategy s = StrategyFactory.getInstance().getStrategy(strategy);
		return s;
	}
	
	private IStrategy gatherStrategyConfig(Mastermind m) {
		
		IStrategy s = getSelectedStrategy();
		StrategyConfiguration c = new StrategyConfiguration();
		c.numOfColors = m.getColors();
		c.numOfPlaces = m.getPlaces();
		c.numOfRows = m.getRows();
		c.withRepetition = m.isWithRepetition();
		
		c.specialConf = mCurrentSpecialStrategyConf;
		s.setConfig(c);
		return s;
	}
	
	private MyEvaluator gatherEvaluatorConfig(Mastermind m, IStrategy s) {
		
		boolean isRandomGames = mRunModeRandom.isChecked();
		int numOfRandomGames = Integer.parseInt(mNumOfRandomGames.getText().toString());
		MyEvaluator eval = new MyEvaluator(m, s, isRandomGames, numOfRandomGames);
		return eval;
	}
	
	
	private void startEvaluation(MyEvaluator e) {
		BoardInfo bi = e.getBoardInfo();
		if (!Utils.isValidCPRCombination(bi.numOfColors, bi.numOfPlaces, bi.withRepetition)) {
			showDialog(INVALID_CONFIGURATION);
			return;
		}
		mE = e;
		
		showDialog(PROGRESS_DIALOG);
	}
	
	private void updateEvaluationResults() {
		Stats stats = mE.getStats();
		StringBuffer sb = new StringBuffer(stats.toString());
		sb.append("\n+++++++++++\n");
		sb.append(mE.getStrategyInfo());
		mResults.setText(sb.toString());
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (data == null)
			return;
		String[] s = data.getStringArrayExtra(Evaluator.INTENT_DATA_NAMES);
		int[] v = data.getIntArrayExtra(Evaluator.INTENT_DATA_VALUES);
		if (s.length != v.length)
			throw new RuntimeException("Evaluator.onActivityResult: s and v have different lengths.");
		mCurrentSpecialStrategyConf = new HashMap<String, Object>();
		for (int i=0; i<s.length; i++)
			mCurrentSpecialStrategyConf.put(s[i], v[i]);
	}
	
	protected Dialog onCreateDialog(int id) {
        switch(id) {
        case PROGRESS_DIALOG:
            progressDialog = new ProgressDialog(Evaluator.this);
            progressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
            progressDialog.setMessage("Playing...");
            progressDialog.setCancelable(true);
            progressDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
				
				public void onCancel(DialogInterface dialog) {
					progressThread.setState(ProgressThread.STATE_DONE);
					progressThread.interrupt();
				}
			});
            return progressDialog;
        case INVALID_CONFIGURATION:
        	invalidConfAlertDialog = new AlertDialog.Builder(Evaluator.this).create();
        	invalidConfAlertDialog.setTitle("Invalid configuration.");
        	invalidConfAlertDialog.setMessage("The parameters you specified are not valid. Please recheck.");
        	invalidConfAlertDialog.setButton("OK", new DialogInterface.OnClickListener() {
				
				public void onClick(DialogInterface dialog, int which) {
					dismissDialog(INVALID_CONFIGURATION);
				}
			});
        	return invalidConfAlertDialog;
        default:
            return null;
        }
    }

	@Override
    protected void onPrepareDialog(int id, Dialog dialog) {
        switch(id) {
        case PROGRESS_DIALOG:
            progressDialog.setProgress(0);
            progressThread = new ProgressThread(handler /*, mE */);
            progressThread.start();
        }
	}

    // Define the Handler that receives messages from the thread and update the progress
    final Handler handler = new Handler() {
        public void handleMessage(Message msg) {
            int total = msg.arg1;
            int all = msg.arg2;
            progressDialog.setMax(mE.totalNumberOfGames());
            if (total == -1 || all == -1) {
            	dismissDialog(PROGRESS_DIALOG);
                progressThread.setState(ProgressThread.STATE_DONE);
                updateEvaluationResults();
            }
            progressDialog.setProgress(total);
            if (total > all){
                dismissDialog(PROGRESS_DIALOG);
                progressThread.setState(ProgressThread.STATE_DONE);
            }
        }
    };

    /** Nested class that performs progress calculations (counting) */
    class ProgressThread extends Thread {
        Handler mHandler;
        final static int STATE_DONE = 0;
        final static int STATE_RUNNING = 1;
        int mState;
        int total;
        int mTotalNumberOfGames;
        // MyEvaluator mEvaluator;
        
        ProgressThread(Handler h /*, MyEvaluator e */) {
            mHandler = h;
            // mEvaluator = e;
        }
       
        public void run() {
            mState = STATE_RUNNING;   
            total = 0;
            // mTotalNumberOfGames = mEvaluator.totalNumberOfGames();
            mTotalNumberOfGames = mE.totalNumberOfGames();
            System.out.println("Total number of games: " + mTotalNumberOfGames);
            while (mState == STATE_RUNNING && mE.hasMoreGames()) {
            	 Message msg = mHandler.obtainMessage();
                 msg.arg1 = total;
                 msg.arg2 = mTotalNumberOfGames;
                 mHandler.sendMessage(msg);
                //try {
                // mEvaluator.playNext();
            	mE.playNext();
                // Thread.sleep(100);
                //} catch (InterruptedException e) {
                //    Log.e("ERROR", "Thread Interrupted");
               // }
               
                total++;
            }
            Message msg = mHandler.obtainMessage();
            msg.arg1 = -1;
            msg.arg2 = -1;
            mHandler.sendMessage(msg);
        }
        
        /* sets the current state for the thread,
         * used to stop the thread */
        public void setState(int state) {
            mState = state;
        }
    }
}
