package de.uni_passau.fim;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import de.uni_passau.fim.gui.ColorRadioButton;


/**
 * This class displays the game activity. The layout of the
 * game is provided in the file <code>res/layout/main.xml</code>.
 */
public class MasterMind extends Activity {

	//*************************************************************************
	// Global variables
	//*************************************************************************
	
	
	//*************************************************************************
	/* These numbers represent the parameters of the game.
	 * They are assigned dynamically in the onCreate(Bundle) below. */
	
	/** Number of colors the ColorSelector offers */
	private int numberOfColors;
	
	/** This is the number of possible tries. */
	private int numberOfRows;
	
	/** The number of colors that make one guess. */
	private int numberOfPlaces;
	
	//*************************************************************************
	/* The following variables represent the current state of the game. */
	
	/** The quest is the color sequence which has to be found by the player. */
	private int[] quest;
	
	/** This is the color that is currently selected by the ColorSelector. */
	private int currentColor;
	
	/** The row of the board that has to be filled with the guess+answer.  */
	private int currentRow;
	
	/** The guess contains the currently set colors of the ColorSetter for easy access. */
	private int guess[];
	
	/** The colors of the ColorSelector are collected in this array. */
	private int[] colors;
	
	/* These arrays could provide an internal representation of the game, similar to the guess array above.
	 * This is useful to compute the hint. */ 
	private int[][] guesses;
	
	private int[][] answers;
	
	private int[] lastGivenHint = null;
	
	/* Use these colors to set the answer fields. */
	
	/** Color of an empty place in the answers. */
	private final int colorUnused = 0xff000000;
	
	/** Color of an place in the answers. */
	private final int colorExact = 0xffff0000;
	
	/** Color of an empty place in the answers. */
	private final int colorCorrect = 0xffffffff;
	
	
	/** The ID of the 'you lost'-dialog */
	private final int DIALOG_GAMEOVER_ID = 0;
	
	/** The ID of the 'you win'-dialog */
	private final int DIALOG_CORRECT_ID = 1;
	
	
	/** This map tells us if the given ImageView (= circle of the ColorSetter)
	 * is the first, second, ...
	 * Access to a Map is provided by the methods put(...) and get(...). */
	private Map<ImageView, Integer> colorSetterMap;
	
	/** This array provides an easy access to the (n+1)-th row of the 'Board' in 
	 * 'main.xml'. It is filled in onCreate(Bundle) below.  */
	private ViewGroup[] boardRowMap;

	/** Static reference to the multiplayer class if
	   we are in multiplayer mode. */
	private static Multiplayer mpClass;

	
	//*************************************************************************
	// Public getter/setter
	//*************************************************************************
	
	/**
	 * Getter for mpClass
	 * @return Multiplayer
	 */
	public static Multiplayer getMpClass() {
		return mpClass;
	}

	/**
	 * Setter for mpClass
	 * @param mpClass the partent MPClass
	 */
	public static void setMpClass(Multiplayer mpClass) {
		MasterMind.mpClass = mpClass;
	}
	
	//*************************************************************************
	// LISTENERS
	//*************************************************************************
	
	/** This listener is assigned to the ColorSelector. It is called if a new
	 * color is selected and it updates the 'currentColor'. */
	private OnCheckedChangeListener colorSelectorListener = new OnCheckedChangeListener() {

		public void onCheckedChanged(RadioGroup group, int checkedId) {
			View v = findViewById(checkedId);
			if (v instanceof ColorRadioButton) {
				currentColor = ((ColorRadioButton) v).getColor();
			}
		}
	};
	
	/** This listener is assigned to the fields of the 'ColorSetter'.
	 * On click it assigns the currently selected color 'currentColor'
	 * to the field. */
	private OnClickListener colorSetterListener = new OnClickListener() {

		public void onClick(View v) {

			if (v instanceof ImageView) {
				// TODO: Aufgabe 2 b): Assign the color of currentColor to the ImageView.
				//       Hint: You can use the helper method 'setColor'
				setColor((ImageView) v, currentColor);
				guess[colorSetterMap.get(v)] = currentColor;
				System.out.println("guess: " + Arrays.toString(guess));
			}
			// TODO: Aufgabe 4 a): validate if we can enable the check button
			boolean valid = true;
			for (int i = 0; valid && i < guess.length; i++) {
				valid = guess[i] != -1;
			}
			if (valid) {
				((Button) findViewById(R.id.CheckButton)).setEnabled(true);
			}
		}
	};

	/*  
	 * TODO Aufgabe 2a) Add this listener to the "Check"-button, see below in method onCreate(Bundle). 
	 */
	/** The listener for the "Check"-button. */
	private OnClickListener checkButtonListener = new OnClickListener() {
		public void onClick(View button) {
			if (mpClass == null) {
				updateBoard(compare(guess, quest));
			} else {
				// we send the guess
				mpClass.sendGuess(guess);
			}
		}
	};
	
	/** The listener for the "Hint"-button. */
	private OnClickListener hintButtonListener = new OnClickListener() {
		public void onClick(View button) {
			computeAndSetHint();
		}
	};


	
	//*************************************************************************
	// onCreate methods (Activity, Dialogs)
	//*************************************************************************

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		
		currentColor = Color.WHITE;

		
		// First collect the colors provided by 'ColorSelector'
		// to the 'colors' array.
		RadioGroup rg = (RadioGroup) this.findViewById(R.id.ColorSelector);
		rg.setOnCheckedChangeListener(colorSelectorListener);
		numberOfColors = rg.getChildCount();
		colors = new int[numberOfColors];
		for (int i = 0; i < rg.getChildCount(); i++) {
			View v = rg.getChildAt(i);
			if (v instanceof ColorRadioButton) {
				colors[i] = ((ColorRadioButton) v).getColor();
			}
		}
		
		// Now handle the 'ColorSetter'. This gives us the number of places.
		ViewGroup current = (ViewGroup) this.findViewById(R.id.ColorSetter);
		colorSetterMap = new HashMap<ImageView, Integer>();
		int placesCounter = 0;
		for (int i = 0; i < current.getChildCount(); i++) {
			View v = current.getChildAt(i);
			if (v instanceof ImageView) {
				v.setOnClickListener(colorSetterListener);
				colorSetterMap.put((ImageView) v, i);
				placesCounter++;
			}
		}
		numberOfPlaces = placesCounter;
		
		// Handle the 'Board' and get the 'numberOfRows'.
		ViewGroup board = (ViewGroup) findViewById(R.id.Board);
		numberOfRows = board.getChildCount();
		boardRowMap = new ViewGroup[numberOfRows];
		for (int j = 0; j < numberOfRows; j++) {
			View v = board.getChildAt(j);
			if (v instanceof ViewGroup) {
				boardRowMap[j] = (ViewGroup) v;
			}
		}
		
		setUpQuest();
		
		currentRow = 0;
		guesses = new int[numberOfRows][numberOfPlaces];
		for (int i = 0; i < guesses.length; i++) {
			Arrays.fill(guesses[i], -1);
		}
		answers = new int[numberOfRows][2];
		for (int i = 0; i < answers.length; i++) {
			Arrays.fill(answers[i], -1);
		}
		lastGivenHint = null;

	    /* TODO: Aufgabe 2a:
	     *       Assign listener 'checkButtonListener' to the 'check'-Button 
	     *       Do the same for the 'hint' button. */
		((Button) findViewById(R.id.CheckButton))
			.setOnClickListener(checkButtonListener);
		((Button) findViewById(R.id.HintButton))
			.setOnClickListener(hintButtonListener);
		
		guess = new int[numberOfPlaces];
		Arrays.fill(guess, -1); // Set all to "white"
		
		
		
		if (mpClass != null) {
			// We are in multi player mode
			Multiplayer.setMasterMind(this);
			Multiplayer.setColors(colors);
		}
	}
	
	/**
	 * Creates a dialoag based on the given id
	 * 
	 * @param id the dialoag ID
	 * @return the dialog
	 */
	protected Dialog onCreateDialog(int id) {
		Dialog dialog = null;
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		switch (id) {
		case DIALOG_CORRECT_ID:
			// TODO: Aufgabe 4 b): Create an 'you win' dialog!
			builder.setTitle(R.string.end)
			.setMessage(R.string.correct)
			.setCancelable(false)
			.setPositiveButton(R.string.OK,
					new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog,
								int id) {
							MasterMind.this.finish();
						}
					});
				dialog = builder.create();
				break;
			
		case DIALOG_GAMEOVER_ID:
			// TODO: Aufgabe 4 b): This dialog doesn't look nice!
			builder.setTitle(R.string.end)
			.setMessage(R.string.gameover)
			.setCancelable(false)
			.setPositiveButton(R.string.OK,
					new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog,
								int id) {
							MasterMind.this.finish();
						}
					});
			dialog = builder.create();
			break;
		default:
			dialog = null;
		}
		return dialog;
	}
	
	
	//*************************************************************************
	// Game Logic
	//*************************************************************************

	/**
	 * Checks, if the current guess is correct.
	 * If true, it displays the "You win"-dialog. If not the
	 * board displays the new guess and the result. If the 
	 * last row is used, the user loses the game.
	 * 
	 * @param
	 */
	public void updateBoard(int[] bw) {
	    /* TODO: Aufgabe 3
	     * 
	     * a) Get the current colors from R.id.ColorSetter.
	     * b) Check the current guess and compute the answer.
	     * c) Set a new row on the game board (with hints for the player).
	     * d) Check if the game is over.
	     */
		// a)
		System.out.println(Arrays.toString(bw));
		if (bw[0] == 4) {
			showDialog(DIALOG_CORRECT_ID);
			if (mpClass != null)
			{ mpClass.reset(); }
			return;
		}
		
		ViewGroup result = (ViewGroup) boardRowMap[currentRow]
				.findViewById(R.id.result);
		// Set the result for the current row
		int reds = bw[0];
		int whites = bw[1];
		for (int i = 0; i < result.getChildCount(); i++) {
			View v = result.getChildAt(i);
			if (v instanceof ViewGroup) {
				ViewGroup tr = (ViewGroup) v;
				for (int j = 0; j < tr.getChildCount(); j++) {
					View v2 = tr.getChildAt(j);
					if (v2 instanceof ImageView) {
						if (reds > 0) {
							setColor((ImageView) v2, colorExact);
							reds--;
						} else {
							if (whites > 0) {
								setColor((ImageView) v2, colorCorrect);
								whites--;
							}
						}
					}
				}
			}
		}
		
		// Set the current row
		ViewGroup row = (ViewGroup) boardRowMap[currentRow].findViewById(
				R.id.cols);
		for (int i = 0; i < row.getChildCount(); i++) {
			View v = row.getChildAt(i);
			if (v instanceof ImageView) {
				setColor((ImageView) v, guess[i]);
			}
		}
		
		// Set the internal representation
		for (int i=0; i<numberOfPlaces; i++) {
			guesses[currentRow][i] = guess[i];
		}
		answers[currentRow] = bw;
		
		currentRow++;
		if (currentRow == numberOfRows ) {
				showDialog(DIALOG_GAMEOVER_ID);
			if (mpClass != null)
				{ mpClass.reset(); }
		}
		
	}

	/* TODO Strategie-Aufgaben: Compute a hint and set it up in the ColorSetter. */
	private void computeAndSetHint() {
		int[] possibleHint = computeNextPossibleHint(lastGivenHint);
		while (!isConsistent(possibleHint))
			computeNextPossibleHint(possibleHint);
		lastGivenHint = possibleHint;
		
		ViewGroup current = (ViewGroup) findViewById(R.id.ColorSetter);
		
		int tCntr = 0;
		for (int i = 0; i < current.getChildCount(); i++) {
			View v = current.getChildAt(i);
			if (v instanceof ImageView) {
				setColor((ImageView) v, possibleHint[tCntr]);
				guess[tCntr] = possibleHint[tCntr];
				tCntr++;
			}
		}
		// we can safely enable the check button
		((Button) findViewById(R.id.CheckButton)).setEnabled(true);
		
	}
	
	
	//*************************************************************************
	// Helper methods.
	//*************************************************************************

	/**
	 * Assign a color to a image view, i.e. an element of a row.
	 * 
	 * @param image on this view to color is assigned 
	 * @param color the assigned color
	 */
	protected static void setColor(ImageView image, int color) {
		Drawable d = image.getDrawable();
		if (d instanceof GradientDrawable) {
			GradientDrawable gd = (GradientDrawable) image.getDrawable();
			gd.mutate();
			gd.setColor(color);
			gd.invalidateSelf();
		}
	}
	
	/**
	 * Randomly fill the quest array with colors available in the 'colors' array. 
	 * 
	 */
	private void setUpQuest() {
		quest = new int[numberOfPlaces];
		int[] readableQuest = new int[numberOfPlaces];
		Random r = new Random();
		for(int i = 0; i < quest.length; i++) {
			int k = r.nextInt(colors.length);
			quest[i] = colors[k];
			readableQuest[i] = k;
			System.out.println("Setting color " + k);
		}
		System.out.println("The quest (readbl): " + Arrays.toString(readableQuest));
		System.out.println("The quest (colors): " + Arrays.toString(quest));
	}
	
	
	private static class RB {
		public int quest;
		public int guess;
		public RB(int quest, int guess) {
			this.quest = quest;
			this.guess = guess;
		}
	}
	
	/**
	 * Compare guess and 
	 * @param quest
	 * @param guess
	 * @return The array [Red, White]
	 */
	public static int[] compareForMP(int[] quest, int[] guess) {
		
		if (quest.length != guess.length)
			throw new IllegalArgumentException("Arrays have different size!");
		
		// We arrange the colors of quest and guess into an mapping
		// Color -> (# occurences in quest, # occurences in guess).
		
		Map<Integer, RB> colorMapper = new HashMap<Integer, RB>();
		
		int reds = 0;
		for (int i = 0; i < guess.length; i++) {
			int currQuestColor = quest[i];
			int currGuessColor = guess[i];
			if (currQuestColor == currGuessColor)
				reds++;
			// Handle quest first
			if (colorMapper.containsKey(Integer.valueOf(currQuestColor))) {
				colorMapper.get(currQuestColor).quest++;
			} else {
				colorMapper.put(Integer.valueOf(currQuestColor), new RB(1,0));
			}
			// Handle then guess
			if (colorMapper.containsKey(Integer.valueOf(currGuessColor))) {
				colorMapper.get(currGuessColor).guess++;
			} else {
				colorMapper.put(Integer.valueOf(currGuessColor), new RB(0,1));
			}
		}
		
		// Now sum up the minimum number of occurrences for each color.
		int whites = 0;
		for (Iterator<RB> iterator = colorMapper.values().iterator(); iterator.hasNext();) {
			RB rb = (RB) iterator.next();
			whites += Math.min(rb.quest, rb.guess);
		}
		// And subtract the number of "Reds", they have been counted also into white
		whites -= reds;
		
		return new int[] {reds, whites};
	}
	
	/** 
	 * Compare quest and guess.
	 * @return The array [red, white].
	 */
	private int[] compare(int[] guess,int[] quest) {
		// First count the number of "Reds", i.e., the number of correct colors
		// in the correct place.
		int red = 0;
		for (int i=0; i<guess.length; i++) 
			if (guess[i] == quest[i])
				red++;
		
		// Now we count, how often each color is in the quest.
		int[] questColorCount = new int[colors.length];
		Arrays.fill(questColorCount, 0);
		for (int i=0; i<quest.length; i++)
			questColorCount[colorToInt(quest[i])]++;
		
		// Now we count, how often each color is in the guess.
		int[] guessColorCount = new int[colors.length];
		Arrays.fill(guessColorCount, 0);
		for (int i=0; i<quest.length; i++)
			guessColorCount[colorToInt(guess[i])]++;
		
		System.out.println("questColorCount " + Arrays.toString(questColorCount));
		System.out.println("guessColorCount " + Arrays.toString(guessColorCount));
		
		// Finally, we sum for each color the minimum number of appearances.
		int white = 0;
		for (int i=0; i<colors.length; i++)
		 	white += Math.min(guessColorCount[i], questColorCount[i]);
		
		// And subtract the number of "Reds", they have been counted also into white
		white -= red;
		System.out.println("Red " + red + ", white: " + white);
		
		return new int[] {red, white};
	}
	
	/** Given some color in its int-representation, return the colors index it has in the
	 * <code>colors</code> array.
	 * @return The index of the color in the <code>colors</code> array or -1 if the given
	 * color is not present */
	private int colorToInt(int color) {
		for (int i = 0; i < colors.length; i++) {
			if (colors[i] == color)
				return i;
		}
		throw new IllegalArgumentException("Color in question is not defined!");
	}
	
	/**
	 * Check if the given guess <code>pGuess</code> is consistent with
	 * all information in <code>guesses</code> and <code>answers</code>.
	 * 
	 * @param pGuess The color combination to be checked.
	 * @return True, if <code>pGuess</code> is consistent with the board.
	 */
	private boolean isConsistent(int[] pGuess) {
		
		for (int i = 0; i < currentRow; i++) {
			int[] tRes = compare(pGuess, guesses[i]);
			if (tRes[0] != answers[i][0] || tRes[1] != answers[i][1] )
				return false;
		}
		return true;
	}
	
	/**
	 * Given the color combination <code>currentHint</code>, 
	 * @param currentHint
	 * @return
	 */
	private int[] computeNextPossibleHint(int[] currentHint) {
		if (currentHint == null) {
			currentHint = new int[numberOfPlaces];
			Arrays.fill(currentHint, colors[0]);
			return currentHint;
		}
		
		return increment(currentHint, 0);
	}
	
	
	/**
	 * Given the color combination array <code>someHint</code>, compute the next
	 * color combination by incrementing to the next color at array position 
	 * <code>place</code>.
	 *  
	 * @param someHint an array with colors.
	 * @param place the index where to increment the array.
	 * @return
	 */
	private int[] increment(int[] someHint, int place) {
		if (place < 0 || place >= someHint.length)
			throw new IllegalArgumentException("Place index out of bounds.");
		int colorIntToIncrement = colorToInt(someHint[place]);
		if (colorIntToIncrement == numberOfColors-1) {
			someHint[place] = colors[0];
			if (place < someHint.length-1)
				return increment(someHint, ++place);
			else // Start over again
				return increment(someHint, 0);
		} else {
			someHint[place] = colors[colorIntToIncrement+1];
		}
		
		return someHint;
	}
	
}