package de.uni_passau.fim;

import java.util.Arrays;
import java.util.Set;

import de.uni_passau.fim.R;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.Button;

/**
 * The Multiplayer class shows the multi player set up screen and contains the
 * messageHandler which interacts with an instance of the Connector class. If a
 * connection between two phones is established, the phone which initiated the
 * connection becomes the server. The other phone becomes the host. The server
 * starts the challenge activity and after that signals the client that it is
 * ready. The client then opens the MasterMind activity. All guesses are sent to
 * the server for verification.
 * 
 */

public class Multiplayer extends Activity implements View.OnClickListener,
		OnItemClickListener {

	// set debugging mode
	private static final String TAG = "MasterMind";
	// if D is set to true debugging is activated
	private static final boolean D = true;

	// Message types sent from the Connection Handler
	public static final int MESSAGE_STATE_CHANGE = 1;
	public static final int MESSAGE_READ = 2;
	public static final int MESSAGE_WRITE = 3;
	public static final int MESSAGE_DEVICE_NAME = 4;
	public static final int MESSAGE_TOAST = 5;

	public static final String DEVICE_NAME = "device_name";
	public static final String TOAST = "toast";

	// Name of the connected device
	private String mConnectedDeviceName = null;

	// GUI elements
	private ArrayAdapter<String> pairedAdapter;
	private ArrayAdapter<String> foundAdapter;
	private BluetoothAdapter blueAdapt;
	private Button startBTSearch;
	private TextView btStatus;
	private ListView pairedDevs;
	private ListView foundDevs;

	// an instance of the Class which handles the threaded connection
	private Connector mConnector = null;

	// here we store the state which we read from Challenge.java
	private static int[] challengeInt;
	
	// stores the colors that are currently in use
	private static int[] colors;

	/**
	 * Getter for colors
	 * @return
	 */
	public static int[] getColors() {
		return colors;
	}

	/**
	 * Setter for colors
	 * @param colors
	 */
	public static void setColors(int[] colors) {
		Multiplayer.colors = colors;
	}

	// here we store a reference to the masterMind multi player activity
	private static MasterMind masterMind;
	

	/**
	 * Getter for challengeInt
	 * 
	 * @return int[]
	 */
	public static int[] getChallengeInt() {
		return challengeInt;
	}

	/**
	 * Setter for challengeInt
	 * 
	 * @param challengeInt
	 *            The challenge for the remote client
	 */
	public static void setChallengeInt(int[] challengeInt) {
		Multiplayer.challengeInt = challengeInt;
		Log.d("Test", "Set MUltiplayer to " + challengeInt[0]);
	}

	/**
	 * Getter for MasterMind multi player instance
	 * 
	 * @return MasterMind
	 */
	public static MasterMind getMasterMind() {
		return masterMind;
	}

	/**
	 * Setter for MasterMind multi player instance
	 * 
	 * @param masterMind
	 */
	public static void setMasterMind(MasterMind masterMind) {
		Multiplayer.masterMind = masterMind;
	}

	/**
	 * Determines if we are client or server
	 * 
	 */

	private static boolean client = true;

	/**
	 * Getter for client
	 * 
	 * @return true/false
	 */
	public static boolean isClient() {
		return client;
	}

	/**
	 * Setter for client
	 * 
	 * @param client
	 *            true/false
	 */
	public static void setClient(boolean client) {
		Log.d("Challenge", "I am the client" + client);
		Multiplayer.client = client;
	}
	
	/**
	 * Resets the connection between two deviced
	 */
	public void reset()
	{
		// shutdown the current connection and start a new one
		try {
			Thread.sleep(200);
			mConnector.stop();
			Thread.sleep(100);
			mConnector.start();
			// also reset the client status
			client = true;
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.multiplayer);

		btStatus = (TextView) findViewById(R.id.btStatus);

		pairedDevs = (ListView) findViewById(R.id.btPairdDevs);
		pairedDevs.setOnItemClickListener(this);

		startBTSearch = (Button) findViewById(R.id.searchDev);
		startBTSearch.setOnClickListener(this);

		foundDevs = (ListView) findViewById(R.id.btFoundDevs);
		foundDevs.setOnItemClickListener(this);

		blueAdapt = BluetoothAdapter.getDefaultAdapter();

		if (blueAdapt != null) {

			// check the number of paired devices

			Set<BluetoothDevice> pairedDevices = blueAdapt.getBondedDevices();
			pairedAdapter = new ArrayAdapter<String>(this,
					android.R.layout.simple_list_item_1);
			foundAdapter = new ArrayAdapter<String>(this,
					android.R.layout.simple_list_item_1);

			// If there are paired devices
			if (pairedDevices.size() > 0) {
				// Loop through paired devices
				for (BluetoothDevice device : pairedDevices) {
					// Add the name and address to an array adapter to show in a
					// ListView
					pairedAdapter.add(device.getName() + "\n"
							+ device.getAddress());
				}
				pairedDevs.setAdapter(pairedAdapter);
			}

			IntentFilter filterFound = new IntentFilter(
					BluetoothDevice.ACTION_FOUND);
			IntentFilter filterDiscovery = new IntentFilter(
					BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
			registerReceiver(mReceiver, filterFound);
			registerReceiver(mReceiver, filterDiscovery);

			// initialize the communication class
			mConnector = new Connector(this, mHandler);
		
			MasterMind.setMpClass(this);

			foundDevs.setAdapter(foundAdapter);

			startBTSearch.setEnabled(true);
			
			// make device discoverable
			Intent discoverableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
			// 300 seconds is the maximum
			discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
			startActivity(discoverableIntent);
			
			// just to be sure try to stop all currently active threads
			mConnector.stop();
			// listen for incoming connect requests
			mConnector.start();
		}

	}

	/**
	 * Create a BroadcastReceiver for ACTION_FOUND and ACTION_DISCOVERY_FINISHED
	 * In case we find a new device we add it to our ArrayAdapter foundAdapter
	 * And in case the scanning for visible Bluetooth devices is over we change
	 * the text on the startBTSearch button
	 */
	private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			// When discovery finds a device
			if (BluetoothDevice.ACTION_FOUND.equals(action)) {
				// Get the BluetoothDevice object from the intent
				BluetoothDevice device = intent
						.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
				// Add the name and address to an array adapter to show in a
				// ListView

				foundAdapter.add(device.getName() + "\n" + device.getAddress());
			} else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED
					.equals(action)) {
				startBTSearch.setText("Search for devices");
			}
		}
	};

	/**
	 * Handles component interaction
	 * 
	 * @param v
	 *            The object which has been clicked
	 */
	public void onClick(View v) {
		if (startBTSearch.equals(v)
				&& startBTSearch.getText().equals("Search for devices")) {
			foundAdapter.clear();
			blueAdapt.startDiscovery();
			startBTSearch.setText("Cancel search");
		} else if (startBTSearch.equals(v)
				&& startBTSearch.getText().equals("Cancel search")) {
			blueAdapt.cancelDiscovery();
			startBTSearch.setText("Search for devices");

		}

	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		try {
			unregisterReceiver(mReceiver);
		} catch (IllegalArgumentException e) {
			// we are in the emulator nothing to do
		}

	}

	/**
	 * If the user has clicked on either a paired or a newly found device we try
	 * to establish a connection. ToDO: The handling could be improved by
	 * creating a wait screen till the connection is fully established.
	 */
	public void onItemClick(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
		if (pairedDevs.equals(arg0) || foundDevs.equals(arg0)) {
			// we are trying to connect so cancel discovery
			// as performance will be bad otherwise
			blueAdapt.cancelDiscovery();

			// the last 17 characters contain the MAC address
			String info = ((TextView) arg1).getText().toString();
			String macAddress = info.substring(info.length() - 17);

			btStatus.setText("Connecting to :" + macAddress);

			BluetoothDevice device = blueAdapt.getRemoteDevice(macAddress);
			
			// Stop listening for an incoming connection
			mConnector.stop();

			// Attempt to connect to the device
			mConnector.connect(device);

			// this side will become the server
			// so let the user input a solution
			client = false;

			// we let the Challenge activity know who called it
			Challenge.setMpClass(this);
			
			// we present the input screen for the challenge
			while (mConnector.getState() == mConnector.STATE_CONNECTING)
			{ try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				e.printStackTrace();
			} }
			
			if (mConnector.getState() == mConnector.STATE_CONNECTED)
			{
				Intent inputChallenge = new Intent();
				inputChallenge.setClassName("de.uni_passau.fim",
						"de.uni_passau.fim.Challenge");
				startActivity(inputChallenge);
			}

		}

	}

	/**
	 * Signal the client that we have selected our challenge via the Connector
	 * class
	 */
	public void signalReady() {
		byte[] send = "ready".getBytes();
		mConnector.write(send);
	}

	/**
	 * Sends the guess to the server via the Connector class
	 * 
	 * @param gS
	 *            int[] guess
	 */
	public void sendGuess(int[] gS) {
		byte[] send =  Arrays.toString(gS).getBytes();
		Log.d("MatserMind", Arrays.toString(gS));
		mConnector.write(send);

	}

	/**
	 * Helper method to parse a string into an int array This can be seen as the
	 * reverse function of int[].toString()
	 * 
	 * @param input
	 *            String
	 * @return the int array
	 */
	public int[] transformToState(String input) {
		
		input = input.substring(1, input.length() - 1);
			
		String[] temp = input.split(",");
		int[] result = new int[temp.length];
		for (int i = 0; i < temp.length; ++i) {
			result[i] = Integer.valueOf(temp[i].trim());
		}

		return result;


	}

	// The Handler that gets information back from the Connector
	public final Handler mHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MESSAGE_STATE_CHANGE:
				if (D)
					Log.i(TAG, "MESSAGE_STATE_CHANGE: " + msg.arg1);
				switch (msg.arg1) {
				case Connector.STATE_CONNECTED:
					break;
				case Connector.STATE_CONNECTING:
					break;
				case Connector.STATE_LISTEN:
					break;
				case Connector.STATE_NONE:
					break;
				}
				break;
			case MESSAGE_WRITE:
				break;
			case MESSAGE_READ:
				byte[] readBuf = (byte[]) msg.obj;
				// construct a string from the valid bytes in the buffer
				String readMessage = new String(readBuf, 0, msg.arg1);
				Log.d("MasterMind","Received string "+ readMessage);
				// we check which kind of message it is
				if (readMessage.equals("ready")) {
					// we open the common GUI
					// which knows that we are in MP mode
					Intent multiPlayerIntent = new Intent();
					multiPlayerIntent.setClassName("de.uni_passau.fim",
							"de.uni_passau.fim.MasterMind");
					startActivity(multiPlayerIntent);

				} else if (readMessage.equals("")) {
					// nothing todo here
				}	else {
					if (client) {
						// first we have to parse the result which tells us
						// what was right or wrong
						int[] localCorrectedResult = transformToState(readMessage);
						masterMind.updateBoard(localCorrectedResult);
					} else {
						// in this section we want to check the guess
						// first we have to parse it
						int[] localGuess = transformToState(readMessage);
						//Model localModel = new Model(challengeInt.length, colors, challengeInt);
						//int[] resultInt = localModel.checkGuess(localGuess);
						System.out.println("localGuess : " + Arrays.toString(localGuess));
						System.out.println("challengeInt : " + Arrays.toString(challengeInt));
						int[] resultInt = MasterMind.compareForMP(localGuess, challengeInt);
						String result = Arrays.toString(resultInt);
						System.out.println("Result: " + result);
						byte[] send = result.getBytes();
						mConnector.write(send);
						
						// if the other player has won we reset the connection
						// ToDO: Display a dialog that states what has happend
						if (resultInt[0] == 4) 	{
							reset();
						}
					}
				}
				break;
			case MESSAGE_DEVICE_NAME:
				// Display the connected devices name
				mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
				Toast.makeText(getApplicationContext(), mConnectedDeviceName,
						Toast.LENGTH_SHORT).show();
				break;
			case MESSAGE_TOAST:
				Toast.makeText(getApplicationContext(),
						msg.getData().getString(TOAST), Toast.LENGTH_SHORT)
						.show();
				break;
			}
		}
	};

}