package de.uni_passau.fim.evaluator;
import java.util.LinkedList;
import java.util.List;

import de.uni_passau.fim.strategies.Answer;
import de.uni_passau.fim.strategies.Configuration;
import de.uni_passau.fim.strategies.Guess;
import de.uni_passau.fim.strategies.Utils;


public class Mastermind {

	private int colors, places, rows;
	
	private boolean withRepetition = true;
	
	private Guess hiddenCode;
	
	private List<Configuration> currentState;
	
	private boolean solved = false;
	
	public Mastermind(int colors, int places, int rows) {
		this.colors = colors;
		this.places = places;
		this.withRepetition = true;
		this.rows = rows;
	}
	
	public Mastermind(int colors, int places, boolean withRepetition, int rows) {
		this.colors = colors;
		this.places = places;
		this.rows = rows;
		this.withRepetition = withRepetition;
	}
	
	public void init() {
		hiddenCode = Utils.createRandomGuess(colors, places, withRepetition);
		currentState = new LinkedList<Configuration>();
		solved = false;
	}
	
	public void init(Guess hiddenCode) {
		this.hiddenCode = hiddenCode;
		currentState = new LinkedList<Configuration>();
		solved = false;
	}
	
	public Answer guess(Guess g) {
		Answer a = g.computeAnswer(hiddenCode);
		solved = (a.getBlack() == places);
		currentState.add(new Configuration(g, a));
		return a;
	}
	
	public boolean solved() {
		return solved;
	}
	
	public List<Configuration> getCurrentState() {
		return currentState;
	}
	
	public int getColors() {
		return colors;
	}
	
	public int getPlaces() {
		return places;
	}
	
	public boolean isWithRepetition() {
		return withRepetition;
	}
	
	public int getRows() {
		return this.rows;
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("*********************************************\n");
		sb.append("* Colors: " + colors + ", places: " + places + ", rows: " + rows +  "\n");
		sb.append("* Hidden code: " + hiddenCode + "\n");
		sb.append("* Current state:\n");
		for (Configuration c : currentState) {
			sb.append("* " + c.toString());
			sb.append("\n");
		}
		sb.append("*********************************************\n");
		return sb.toString();
	}
}
