package de.uni_passau.fim.strategies;

/** A Configuration represents one "evaluated" row of the Mastermind board 
 * containing a {@link Guess} and the corresponding {@link Answer}.
 */
public class Configuration {
	private Guess guess;
	private Answer answer;
	
	private volatile int hashCode;
	
	/** Create a new configuration with the given {@link Guess} and
	 * {@link Answer}. */
	public Configuration(Guess g, Answer a) {
		guess = g;
		answer = a;
		int tmpHash = 17;
		tmpHash = 37*tmpHash + g.hashCode();
		tmpHash = 37*tmpHash + a.hashCode();
		hashCode = tmpHash;
	}
	
	/** Returns a string representation of the form
	 * <code>[1,1,2,2] (2,0)</code> for the guessed code 
	 * <code>[1,1,2,2]</code> and the answer <code>(2,0)</code>. */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(guess.toString());
		sb.append(" ");
		sb.append(answer.toString());
		return sb.toString();
	}
	
	/** Returns <code>true</code> only if <code>o</code> is another
	 * <code>Configuration</code> with the same guess and answer. */
	public boolean equals(Object c) {
		if (this == c)
			return true;
		if (! (c instanceof Configuration))
			return false;
		Configuration g = (Configuration)c;
		return this.guess.equals(g.guess) && this.answer.equals(g.answer);
	}
	
	public int hashCode() {
		return hashCode;
	}
	
	public Guess getGuess() {
		return guess;
	}
	
	public Answer getAnswer() {
		return answer;
	}
}
