package de.uni_passau.fim.strategies;

import java.util.HashMap;
import java.util.List;

/**
 * Interface for strategies. Note that strategies do not have to be
 * deterministic, i.e., it is possible that <code>getGuess</code> returns
 * two different {@link Guess}es for two calls with the same {@link Configuration}.
 * 
 * A strategy can have an internal state that depends on previous calls of
 * <code>getGuess()</code> which can be reset calling <code>reset()</code>.
 */

public interface IStrategy {

	/** Provide a {@link Guess} depending on the currently given {@link Configuration}. */
	public Guess getGuess(List<Configuration> configs);
	
	public void setConfig(StrategyConfiguration c);
	
	/** Reset the internal state. */
	public void reset();
	
	/** A string containing information on the strategy, its current
	 * internal state, configuration etc. */
	public String info();
	
	/** For identification, is also shown in the Spinner of the Strategy evaluation part. */
	public String getUniqueName();
	
	/** May return null if no extrat configuration is needed. */
	public HashMap<String, Object> getDefaultSpecialConfig();
	
}
